package com.framsticks.gui;

import javax.swing.JFrame;

import org.apache.log4j.Logger;
import org.fest.swing.core.Robot;
import org.fest.swing.edt.FailOnThreadViolationRepaintManager;
import org.fest.swing.edt.GuiQuery;
import org.fest.swing.fixture.FrameFixture;
import org.fest.swing.fixture.JTreeFixture;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.framsticks.test.TestConfiguration;
import com.framsticks.util.dispatching.Monitor;

import static org.fest.assertions.Assertions.assertThat;
import static org.fest.swing.edt.GuiActionRunner.executeInEDT;
import static org.fest.swing.core.BasicRobot.robotWithNewAwtHierarchy;
import org.fest.swing.edt.GuiActionRunner;

@Test
public abstract class BrowserBaseTest extends TestConfiguration {

	private static final Logger log = Logger.getLogger(BrowserTest.class);

	protected Monitor monitor;
	protected Browser browser;
	protected static Robot robot;
	protected FrameFixture frame;
	protected JTreeFixture tree;

	static {
		FailOnThreadViolationRepaintManager.install();
		assertThat(executeInEDT()).isTrue();

		robot = robotWithNewAwtHierarchy();
	}

	@BeforeClass(timeOut = 5000)
	public void setUp() {

		configureBrowser();

		monitor = new Monitor(browser);

		monitor.use();

		frame = new FrameFixture(robot,
				GuiActionRunner.execute(new GuiQuery<JFrame>() {
					@Override
					protected JFrame executeInEDT() throws Throwable {
						return browser.getMainFrame().getSwing();
					}
				}));

		tree = frame.tree("tree");
		log.debug("frame fixture done");
	}

	protected abstract void configureBrowser();

	protected void clickAndExpandPath(String path) {
		tree.clickPath(path).expandPath(path);
		robot.waitForIdle();
	}

	protected void waitForIdle() {
		robot.waitForIdle();
	}

	@AfterClass(timeOut = 5000)
	public void tearDown() {
		monitor.drop().join();
	}

}
