package com.framsticks.gui;

import static org.fest.assertions.Assertions.assertThat;

import org.testng.annotations.Test;

import com.framsticks.core.Instance;
import com.framsticks.core.ObjectInstance;
import com.framsticks.model.Model;
import com.framsticks.params.types.FloatParam;
import com.framsticks.parsers.XmlLoader;
import com.framsticks.util.dispatching.RunAt;

@Test
public class ObjectInstanceBrowserTest extends BrowserBaseTest {

	ObjectInstance instance;

	@Override
	protected void configureBrowser() {
		browser = new XmlLoader().load(Browser.class, getClass().getResourceAsStream("/configs/ObjectInstanceBrowserTest.xml"));

		assertThat(browser.getInstances().size()).isEqualTo(1);
		assertThat(browser.getInstances().get("model")).isInstanceOf(ObjectInstance.class);

		instance = (ObjectInstance) browser.getInstances().get("model");
	}

	@Test(timeOut = 10000)
	public void testShow() {
		assertDispatch(instance, new RunAt<Instance>() {
			@Override
			public void run() {
				assertThat(instance.getRootObject()).isInstanceOf(Model.class);
			}
		});

		clickAndExpandPath("model");
		clickAndExpandPath("model/parts");
		clickAndExpandPath("model/parts/2");

		assertDispatch(instance, new RunAt<Instance>() {
			@Override
			public void run() {
				assertThat(instance.bindAccess("/parts/2").getFramsClass().getParamEntry("m", FloatParam.class).getMax(Double.class)).isEqualTo(999.0);
			}
		});

	}


}
