package com.framsticks.test;

import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.testng.annotations.*;

import com.framsticks.util.dispatching.Dispatcher;
import com.framsticks.util.dispatching.RunAt;

public class TestConfiguration {

	private static final Logger log = Logger.getLogger(TestConfiguration.class);

	@BeforeClass
	public void setUpConfiguration() {
		PropertyConfigurator.configure(TestConfiguration.class.getResource("/log4j.properties"));
		log.info("testing " + this.getClass());
	}

	private final List<AsyncAssert<?>> asyncAssertions = new LinkedList<AsyncAssert<?>>();

	public class AsyncAssert<C> extends RunAt<C> {
		final RunAt<? extends C> runnable;
		boolean done = false;
		AssertionError assertion;

		/**
		 * @param runnable
		 */
		public AsyncAssert(RunAt<? extends C> runnable) {
			this.runnable = runnable;
		}

		@Override
		public void run() {
			try {
				runnable.run();
			} catch (AssertionError e) {
				assertion = e;
			}
			synchronized (this) {
				done = true;
				this.notifyAll();
			}
		}
	}

	public <C> void assertDispatch(Dispatcher<C> dispatcher, RunAt<? extends C> runnable) {
		AsyncAssert<C> assertion = new AsyncAssert<C>(runnable);
		synchronized (asyncAssertions) {
			asyncAssertions.add(assertion);
		}
		dispatcher.dispatch(assertion);
	}

	@BeforeMethod
	public void clearAsyncAsserts() {
		synchronized (asyncAssertions) {
			asyncAssertions.clear();
		}
	}

	@AfterMethod(timeOut = 5000)
	public void waitForAsyncAsserts() {
		while (true) {
			AsyncAssert<?> assertion;
			synchronized (asyncAssertions) {
				if (asyncAssertions.isEmpty()) {
					return;
				}
				assertion = asyncAssertions.get(0);
				asyncAssertions.remove(0);
			}
			synchronized (assertion) {
				while (!assertion.done) {
					try {
						assertion.wait();
					} catch (InterruptedException ignored) {
					}
				}
				if (assertion.assertion != null) {
					throw assertion.assertion;
				}
			}
		}
	}
}
